@@ -4,7 +4,7 @@ package Mail::AuthenticationResults::Header::Base;
 require 5.008;
 use strict;
 use warnings;
-our $VERSION = '1.20180518'; # VERSION
+our $VERSION = '1.20180923'; # VERSION
 use Scalar::Util qw{ weaken refaddr };
 use Carp;
 
@@ -317,15 +317,48 @@ sub search {
         }
     }
 
+    if ( exists( $search->{ 'authserv_id' } ) ) {
+        if ( $self->_HAS_VALUE() ) {
+            if ( lc ref $self eq 'mail::authenticationresults::header' ) {
+                my $authserv_id = eval{ $self->value()->value() } || q{};
+                if ( ref $search->{ 'authserv_id' } eq 'Regexp' && $authserv_id =~ m/$search->{'authserv_id'}/ ) {
+                    $match = $match && 1;
+                }
+                elsif ( lc $search->{ 'authserv_id' } eq lc $authserv_id ) {
+                    $match = $match && 1;
+                }
+                else {
+                    $match = 0;
+                }
+            }
+            else {
+                $match = 0;
+            }
+        }
+        else {
+            $match = 0; # uncoverable statement
+            # There are no code paths with the current classes which end up here
+        }
+    }
+
     if ( exists( $search->{ 'isa' } ) ) {
         if ( lc ref $self eq 'mail::authenticationresults::header::' . lc $search->{ 'isa' } ) {
             $match = $match && 1;
         }
+        elsif ( lc ref $self eq 'mail::authenticationresults::header' && lc $search->{ 'isa' } eq 'header' ) {
+            $match = $match && 1;
+        }
         else {
             $match = 0;
         }
     }
 
+    if ( exists( $search->{ 'has' } ) ) {
+        foreach my $query ( @{ $search->{ 'has' } } ) {
+            $match = 0 if ( scalar @{ $self->search( $query )->children() } == 0 );
+        }
+    }
+
     if ( $match ) {
         $group->add_child( $self );
     }
@@ -356,7 +389,7 @@ Mail::AuthenticationResults::Header::Base - Base class for modelling parts of th
 
 =head1 VERSION
 
-version 1.20180518
+version 1.20180923
 
 =head1 DESCRIPTION
 
@@ -507,6 +540,11 @@ Match if the instance value matches the supplied value (string or regex)
 Match is the instance class typs matches the supplied value. This is a lowercase version
 of the class type, (comment,entry,subentry,etc))
 
+=item has
+
+An arrayref of searches, match this class if the supplied search queries would return at
+least 1 result each
+
 =back
 
 =head1 AUTHOR
